{*
Delphi Jedi-SDL, palette rotating & simple fading example
Created by Willems Davy - Willems Soft
Http://www.Willemssoft.be

This is just a simple example with 3 extra functions i made
for palette rotating in both directions and a simple fading in
& fading out routing for 8 bit surfaces!

The jedi-sdl libaries and SDL runtime dll's needs to be installed for
this example to work!

http://delphi-jedi.org
*}
program Project1;

uses
  sdl,
  windows,
  sysutils,
  sdlutils;

{$R *.res}

var
screen : PSDL_SURFACE;
quit,movedown : boolean;
Event: TSDL_Event;
Tel : integer;
palette,TargetPalette : Tsdl_colorarray;

function SDL_Fadedown(surface: PSDL_Surface) : boolean;
var
 tel :integer;
 pal : TSDL_ColorArray;
 done : boolean;
begin

 //check if surface has 8 bits per pixel
 if surface.format.BitsPerPixel <> 8 then
 begin
  SDL_Fadedown := false;
  exit;
 end;

 done := true;
 movedown := false;
 
 //Copy the current pallete to work with
 copymemory(@pal,surface.format.palette.colors,256*sizeof(TSDL_COLOR));

 //dec RGB with 1 if it didn't meet the wanted color yet (black)
 for tel := 0 to 255 do
 begin
  if pal[tel].r > 0 then dec(pal[tel].r);
  if pal[tel].g > 0 then dec(pal[tel].g);
  if pal[tel].b > 0 then dec(pal[tel].b);

  //check if fading is done
  if (pal[tel].r > 0) or (pal[tel].g > 0) or (pal[tel].b > 0) then done := false;
 end;

 //update the changed palette on the surface
 SDL_setpalette(surface,sdl_logpal or SDL_PHYSpal,@pal,0,256);

 //return fading done boolean
 SDL_Fadedown := done

end;

function SDL_FadeUp(surface : psdl_surface;TargetPalette : TSDL_ColorArray) : boolean;
var
 tel :integer;
 pal : TSDL_ColorArray;
 done : boolean;
begin
 
 //check if surface has 8 bits per pixel
 if surface.format.BitsPerPixel <> 8 then
 begin
  SDL_FadeUp := false;
  exit;
 end;

 done := true;

 //Copy the current pallete to work with
 copymemory(@pal,surface.format.palette.colors,256*sizeof(TSDL_COLOR));

 //inc RGB with 1 if it didn't meet the wanted color yet
 for tel := 0 to 255 do
 begin
  if pal[tel].r < TargetPalette[tel].r then inc(pal[tel].r);
  if pal[tel].g < TargetPalette[tel].g then inc(pal[tel].g);
  if pal[tel].b < TargetPalette[tel].b then inc(pal[tel].b);

  //check if fading is done
  if (pal[tel].r < TargetPalette[tel].r) or (pal[tel].g < TargetPalette[tel].g) or (pal[tel].b < TargetPalette[tel].b) then done := false;
 end;

 //update the changed palette on the surface
 SDL_setpalette(surface,sdl_logpal or SDL_PHYSpal,@pal,0,256) ;

 //return fading done boolean
 SDL_FadeUp := done;
end;

function SDL_RotatePalette(Surface : PSDL_Surface; Movedown : boolean) : boolean;
var
 Pal : TSDL_ColorArray;
 Temp : TSDL_Color;
 Tel : integer;
begin

 //check if surface has 8 bits per pixel
 if surface.format.BitsPerPixel <> 8 then
 begin
  SDL_RotatePalette := false;
  exit;
 end;

 // Get the current palette from the surface
  CopyMemory(@Pal,Surface.Format.Palette.Colors,256*sizeof(TSDL_Color));

 if Movedown then
 begin
  Temp := Pal[255];
  for Tel := 254 downto  0  do Pal[Tel+1] := Pal[Tel];
  Pal[0] := Temp;
 end
 else
 begin
  Temp := Pal[0];
  for Tel := 0 to 254 do Pal[tel] := Pal[tel+1];
  Pal[255] := Temp;
 end;

 //return true if the pallete has been succesfully set & rotated
 SDL_RotatePalette := (SDL_SetPalette(Surface,SDL_LOGPAL or SDL_PHYSPAL,@Pal,0,256) = 1)
end;

// draw something on the screen
procedure startscherm;
var
x,y : integer;
begin
 for y := 0 TO 99 do
  begin
   for x := 0 TO 159 do
    begin
     SDL_LOCKSURFACE(screen);
     sdl_putpixel (screen,x, y, y + x);
     sdl_putpixel (screen,319 - x,y,y + x);
     sdl_putpixel (screen,x, 199 - y, y + x);
     sdl_putpixel (screen,319 - x, 199 - y, y + x);
     sdl_unlocksurface(screen);
     end;
  end;
  sdl_updaterect(screen,0,0,0,0);
end;



begin
 if SDL_Init(SDL_INIT_VIDEO + SDL_INIT_TIMER) >= 0 then
 begin
  screen := SDL_SetVideoMode(320,200,8,SDL_SWSURFACE or SDL_HWPALETTE  );
  if screen <> nil then
  begin
   //no cursor needed
   SDL_SHOWCURSOR(SDL_DISABLE);

   //set caption
   SDL_WM_SetCaption('SDL Fading & Palette Rotation',nil);

   quit := false;

   //create a black color only pallet
   for tel := 0 to 255 do
   begin
    palette[tel].r := 0 ;
    palette[tel].g := 0 ;
    palette[tel].b := 0 ;
   end;

   //get target pallete (in this case the normal 256 bit pallet created with the surface)
   copymemory(@TargetPalette,screen.format.palette.colors,256*sizeof(TSDL_COLOR));

   //set pallet to all black colors to start fading up
   sdl_Setpalette(screen,SDL_LOGPAL or SDL_PHYSPAL,@palette,0,256);


   //draw our stuff with the black pallete (load image here if u like or whatever u like to fade
   startscherm;

   //as Long as fading up is not finished keep fading up
   while not sdl_fadeup(screen,TargetPalette) do
   begin
    sdl_fadeup(screen,TargetPalette);
    sdl_delay(5);
   end;

   //as long as we didn't press esc to quit, keep rotating the palette
   //and if a key is pressed other then esc, change the rotating direction
   while not quit do
   begin
    while SDL_PollEvent(@Event) > 0 do
     begin
      case Event.key.type_ of
       SDL_QUITEV :
       begin
        Quit := true;
       end;

       SDL_KeyDown :
       begin
        if Event.key.keysym.sym = SDLK_ESCAPE then Quit := true  else movedown := not movedown;
       end;
      end;
     end;

    SDL_RotatePalette(screen,movedown);
    Sdl_delay(5);
   end;



   //as Long as fading down is not finished keep fading down
   while not sdl_fadedown(screen) do
   begin
    sdl_fadedown(screen);
    sdl_delay(5);
   end;
  end
  else // setting screen failed give error
  begin
   messagebox(0,pchar(format('Couldn''t create screen mode 320x240x8 : %s',[SDL_GetError])),'Error', MB_OK+MB_ICONHAND);
  end;

  // quit sdl when all is done
  SDL_Quit;
 end
 else // initializing sdl Failed, give error
 begin
  messagebox(0,pchar(format('Couldn''t initialize SDL : %s',[SDL_GetError])),'Error', MB_OK+MB_ICONHAND);
 end;

end.
